@@ -161,20 +161,31 @@
 #define trace_xfs_perag_get_tag(a,b,c,d) ((c) = (c))
 #define trace_xfs_perag_put(a,b,c,d)	((c) = (c))
 
-#define trace_xfs_defer_init(a,b,c)		((void) 0)
-#define trace_xfs_defer_cancel(a,b,c)		((void) 0)
-#define trace_xfs_defer_intake_work(a,b)	((void) 0)
-#define trace_xfs_defer_intake_cancel(a,b)	((void) 0)
+#define trace_xfs_trans_alloc(a,b)		((void) 0)
+#define trace_xfs_trans_cancel(a,b)		((void) 0)
+#define trace_xfs_trans_brelse(a)		((void) 0)
+#define trace_xfs_trans_binval(a)		((void) 0)
+#define trace_xfs_trans_bjoin(a)		((void) 0)
+#define trace_xfs_trans_bhold(a)		((void) 0)
+#define trace_xfs_trans_get_buf(a)		((void) 0)
+#define trace_xfs_trans_getsb_recur(a)		((void) 0)
+#define trace_xfs_trans_getsb(a)		((void) 0)
+#define trace_xfs_trans_read_buf_recur(a)	((void) 0)
+#define trace_xfs_trans_read_buf(a)		((void) 0)
+#define trace_xfs_trans_commit(a,b)		((void) 0)
+
+#define trace_xfs_defer_cancel(a,b)		((void) 0)
 #define trace_xfs_defer_pending_commit(a,b)	((void) 0)
 #define trace_xfs_defer_pending_abort(a,b)	((void) 0)
-#define trace_xfs_defer_pending_cancel(a,b)	((void) 0)
 #define trace_xfs_defer_pending_finish(a,b)	((void) 0)
-#define trace_xfs_defer_trans_abort(a,b,c)	((void) 0)
-#define trace_xfs_defer_trans_roll(a,b,c)	((void) 0)
-#define trace_xfs_defer_trans_roll_error(a,b,c)	((void) 0)
-#define trace_xfs_defer_finish(a,b,c)		((void) 0)
-#define trace_xfs_defer_finish_error(a,b,c)	((void) 0)
-#define trace_xfs_defer_finish_done(a,b,c)	((void) 0)
+#define trace_xfs_defer_trans_abort(a,b)	((void) 0)
+#define trace_xfs_defer_trans_roll(a,b)		((void) 0)
+#define trace_xfs_defer_trans_roll_error(a,b)	((void) 0)
+#define trace_xfs_defer_finish(a,b)		((void) 0)
+#define trace_xfs_defer_finish_error(a,b)	((void) 0)
+#define trace_xfs_defer_finish_done(a,b)	((void) 0)
+#define trace_xfs_defer_cancel_list(a,b)	((void) 0)
+#define trace_xfs_defer_create_intent(a,b)	((void) 0)
 
 #define trace_xfs_bmap_free_defer(...)		((void) 0)
 #define trace_xfs_bmap_free_deferred(...)	((void) 0)
