@@ -14,7 +14,7 @@
 char	*progname;
 int	exitcode;
 
-void
+static void
 usage(void)
 {
 	fprintf(stderr,
@@ -54,13 +54,13 @@ init_check_command(
 	return 1;
 }
 
-void
+static void
 init(
 	int		argc,
 	char		**argv)
 {
 	int		c;
-	xfs_fsop_geom_t	geometry = { 0 };
+	struct xfs_fsop_geom geometry = { 0 };
 	struct fs_path	fsp;
 
 	progname = basename(argv[0]);
